@@ -4,10 +4,11 @@ sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
 from rmtest import ModuleTestCase
 import redis
 import unittest
-import random
-import time
 import bz2
 import csv
+from io import TextIOWrapper
+
+import six
 
 from redisearch import *
 
@@ -31,30 +32,29 @@ class RedisSearchTestCase(ModuleTestCase('../module.so')):
             return self.createIndex(client, num_docs=num_docs)
 
         chapters = {}
-        
-        with bz2.BZ2File(WILL_PLAY_TEXT) as fp:
-            
-            r = csv.reader(fp, delimiter=';')
-            for n, line in enumerate(r):
-                #['62816', 'Merchant of Venice', '9', '3.2.74', 'PORTIA', "I'll begin it,--Ding, dong, bell."]
+        bzfp = bz2.BZ2File(WILL_PLAY_TEXT)
+        if six.PY3:
+            bzfp = TextIOWrapper(bz2.BZ2File(WILL_PLAY_TEXT), encoding='utf8')
 
-                play, chapter, character, text = line[1], line[2], line[4], line[5]
+        r = csv.reader(bzfp, delimiter=';')
+        for n, line in enumerate(r):
+            #['62816', 'Merchant of Venice', '9', '3.2.74', 'PORTIA', "I'll begin it,--Ding, dong, bell."]
 
-                key = '{}:{}'.format(play, chapter).lower()
-                d = chapters.setdefault(key, {})
-                d['play'] = play
-                d['txt'] = d.get('txt', '') + ' ' + text
-                d['chapter'] = int(chapter or 0)
+            play, chapter, character, text = line[1], line[2], line[4], line[5]
+
+            key = '{}:{}'.format(play, chapter).lower()
+            d = chapters.setdefault(key, {})
+            d['play'] = play
+            d['txt'] = d.get('txt', '') + ' ' + text
+            d['chapter'] = int(chapter or 0)
+            if len(chapters) == num_docs:
+                break
 
-                if len(chapters) == num_docs:
-                    break
-                
         indexer = client.batch_indexer(chunk_size=50)
         self.assertIsInstance(indexer, Client.BatchIndexer)
         self.assertEqual(50, indexer.chunk_size)
 
-        for key, doc in chapters.iteritems():
-            
+        for key, doc in six.iteritems(chapters):
             indexer.add_document(key, **doc)
         indexer.commit()
 
@@ -134,10 +134,8 @@ class RedisSearchTestCase(ModuleTestCase('../module.so')):
                 self.assertEqual(len(subset), docs.total)
                 ids = [x.id for x in docs.docs]
                 self.assertEqual(set(ids), set(subset))
-
-                for doc in client.search(Query('henry king').return_fields('play', 'nonexist')).docs:
-                    self.assertFalse(doc.nonexist)
-                    self.assertTrue(doc.play.startswith('Henry'))
+ 
+#                 self.assertRaises(redis.ResponseError, client.search, Query('henry king').return_fields('play', 'nonexist'))
 
                 # test slop and in order
                 self.assertEqual(193, client.search(Query('henry king')).total)
@@ -196,6 +194,30 @@ class RedisSearchTestCase(ModuleTestCase('../module.so')):
             self.assertEqual('foo baz', res.docs[1].payload)
             self.assertIsNone(res.docs[0].payload)
 
+    def testScores(self):
+        
+        conn = self.redis()
+
+        with conn as r:
+            # Creating a client with a given index name
+            client = Client('idx', port=conn.port)
+            client.redis.flushdb()
+            client.create_index((TextField('txt'),))
+
+            client.add_document('doc1', txt = 'foo baz')
+            client.add_document('doc2', txt = 'foo bar')
+
+            q = Query("foo ~bar").with_scores()
+            res = client.search(q)
+            print("RES", res)
+            self.assertEqual(2, res.total)
+            
+            self.assertEqual('doc2', res.docs[0].id)
+            self.assertEqual(3.0, res.docs[0].score)
+
+            self.assertEqual('doc1', res.docs[1].id)
+            self.assertEqual(0.2, res.docs[1].score)
+
     def testReplace(self):
         
         conn = self.redis()
@@ -354,7 +376,6 @@ class RedisSearchTestCase(ModuleTestCase('../module.so')):
             strs = []
             for _ in r.retry_with_rdb_reload():
                 ret = ac.get_suggestions('bad', with_scores = True)
-
                 self.assertEqual(2, len(ret))
                 self.assertEqual('badger', ret[0].string)
                 self.assertIsInstance(ret[0].score, float)
@@ -439,6 +460,30 @@ class RedisSearchTestCase(ModuleTestCase('../module.so')):
             # values
             res = client.search('@f3:f3_val @f2:f2_val @f1:f1_val')
             self.assertEqual(1, res.total)
+            
+            
+    def testNoCreate(self):
+        client = self.getCleanClient('idx')
+        client.create_index((TextField('f1'), TextField('f2'), TextField('f3')))
+
+        client.add_document('doc1', f1='f1_val', f2='f2_val')
+        client.add_document('doc2', f1='f1_val', f2='f2_val')
+
+        client.add_document('doc1', f3='f3_val', no_create=True)
+        client.add_document('doc2', f3='f3_val', no_create=True, partial=True)
+
+        for i in self.retry_with_reload():
+            # Search for f3 value. All documents should have it
+            res = client.search('@f3:f3_val')
+            self.assertEqual(2, res.total)
+
+            # Only the document updated with PARTIAL should still have the f1 and f2
+            # values
+            res = client.search('@f3:f3_val @f2:f2_val @f1:f1_val')
+            self.assertEqual(1, res.total)            
+            
+        with self.assertRaises(redis.ResponseError) as error:
+            client.add_document('doc3', f2='f2_val', f3='f3_val', no_create=True)       
 
     def testExplain(self):
         client = self.getCleanClient('idx')
@@ -497,6 +542,123 @@ class RedisSearchTestCase(ModuleTestCase('../module.so')):
                 res = client.search(q)
                 self.assertEqual(1, res.total)
 
+    def testTextFieldSortableNostem(self):
+        conn = self.redis()
+
+        with conn as r:
+            # Creating a client with a given index name
+            client = Client('sortableNostem', port=conn.port)
+            client.redis.flushdb()
+
+            # Creating the index definition with sortable and no_stem
+            client.create_index((TextField('txt', sortable=True, no_stem=True),))
+
+            # Now get the index info to confirm its contents
+            response = client.info()
+            self.assertIn(b'SORTABLE', response['fields'][0])
+            self.assertIn(b'NOSTEM', response['fields'][0])
+
+    def testAlterSchemaAdd(self):
+        conn = self.redis()
+
+        with conn as r:
+            # Creating a client with a given index name
+            client = Client('alterIdx', port=conn.port)
+            client.redis.flushdb()
+
+            # Creating the index definition and schema
+            client.create_index((TextField('title'),))
+
+            # Using alter to add a field
+            client.alter_schema_add((TextField('body'),))
+
+            # Indexing a document
+            client.add_document('doc1', title = 'MyTitle', body = 'Some content only in the body')
+
+            # Searching with parameter only in the body (the added field)
+            q = Query("only in the body")
+
+            # Ensure we find the result searching on the added body field
+            res = client.search(q)
+            self.assertEqual(1, res.total)
+
+    def testSpellCheck(self):
+        client = self.getCleanClient('idx')
+        client.create_index((TextField('f1'), TextField('f2')))
+
+        client.add_document('doc1', f1='some valid content', f2='this is sample text')
+        client.add_document('doc2', f1='very important', f2='lorem ipsum')
+
+        for i in self.retry_with_reload():
+            res = client.spellcheck('impornant')
+            self.assertEqual('important', res['impornant'][0]['suggestion'])
+
+            res = client.spellcheck('contnt')
+            self.assertEqual('content', res['contnt'][0]['suggestion'])
+
+    def testDictOps(self):
+        client = self.getCleanClient('idx')
+        client.create_index((TextField('f1'), TextField('f2')))
+
+        for i in self.retry_with_reload():
+            # Add three items
+            res = client.dict_add('custom_dict', 'item1', 'item2', 'item3')
+            self.assertEqual(3, res)
+
+            # Remove one item
+            res = client.dict_del('custom_dict', 'item2')
+            self.assertEqual(1, res)
+
+            # Dump dict and inspect content
+            res = client.dict_dump('custom_dict')
+            self.assertEqual(['item1', 'item3'], res)
+
+            # Remove rest of the items before reload
+            client.dict_del('custom_dict', *res)
+
+    def testPhoneticMatcher(self):
+        conn = self.redis()
+
+        with conn as r:
+            # Creating a client with a given index name
+            client = Client('myIndex', port=conn.port)
+            client.redis.flushdb()
+
+            client.create_index((TextField('name'),))
+
+            client.add_document('doc1', name='Jon')
+            client.add_document('doc2', name='John')
+
+            res = client.search(Query("Jon"))
+            self.assertEqual(1, len(res.docs))
+            self.assertEqual('Jon', res.docs[0].name)
+
+            # Drop and create index with phonetic matcher
+            client.redis.flushdb()
+
+            client.create_index((TextField('name', phonetic_matcher='dm:en'),))
+
+            client.add_document('doc1', name='Jon')
+            client.add_document('doc2', name='John')
+
+            res = client.search(Query("Jon"))
+            self.assertEqual(2, len(res.docs))
+            self.assertEqual(['John', 'Jon'], sorted([d.name for d in res.docs]))
+            
+    def testGet(self):
+        client = self.getCleanClient('idx')
+        client.create_index((TextField('f1'), TextField('f2')))
+
+        self.assertEqual([None], client.get('doc1'))
+        self.assertEqual([None, None], client.get('doc2', 'doc1'))
+        
+        client.add_document('doc1', f1='some valid content dd1', f2='this is sample text ff1')
+        client.add_document('doc2', f1='some valid content dd2', f2='this is sample text ff2')
+        
+        self.assertEqual([['f1', 'some valid content dd2', 'f2', 'this is sample text ff2']], client.get('doc2'))
+        self.assertEqual([['f1', 'some valid content dd1', 'f2', 'this is sample text ff1'], ['f1', 'some valid content dd2', 'f2', 'this is sample text ff2']], client.get('doc1', 'doc2'))
+
+
 if __name__ == '__main__':
 
     unittest.main()
